@@ -1,15 +1,20 @@
+import json
 from datetime import date, datetime
 
 import six
 from django.conf import settings
 from django.core.mail.backends.base import BaseEmailBackend
 from django.utils.timezone import is_naive, get_current_timezone, make_aware, utc
+from requests.structures import CaseInsensitiveDict
 
-from ..exceptions import AnymailCancelSend, AnymailError, AnymailUnsupportedFeature, AnymailRecipientsRefused
+from ..exceptions import (
+    AnymailCancelSend, AnymailError, AnymailUnsupportedFeature, AnymailRecipientsRefused,
+    AnymailSerializationError)
 from ..message import AnymailStatus
 from ..signals import pre_send, post_send
-from ..utils import (Attachment, UNSET, combine, last, get_anymail_setting, parse_address_list,
-                     force_non_lazy, force_non_lazy_list, force_non_lazy_dict, is_lazy)
+from ..utils import (
+    Attachment, UNSET, combine, last, get_anymail_setting, parse_address_list, parse_single_address,
+    force_non_lazy, force_non_lazy_list, force_non_lazy_dict, is_lazy)
 
 
 class AnymailBaseBackend(BaseEmailBackend):
@@ -229,6 +234,7 @@ class BasePayload(object):
     )
     anymail_message_attrs = (
         # Anymail expando-props
+        ('envelope_sender', last, parse_single_address),
         ('metadata', combine, force_non_lazy_dict),
         ('send_at', last, 'aware_datetime'),
         ('tags', combine, force_non_lazy_list),
@@ -262,12 +268,18 @@ class BasePayload(object):
                 if converter is not None:
                     if not callable(converter):
                         converter = getattr(self, converter)
-                    value = converter(value)
+                    if converter in (parse_address_list, parse_single_address):
+                        # hack to include field name in error message
+                        value = converter(value, field=attr)
+                    else:
+                        value = converter(value)
             if value is not UNSET:
                 if attr == 'body':
                     setter = self.set_html_body if message.content_subtype == 'html' else self.set_text_body
                 elif attr == 'from_email':
                     setter = self.set_from_email_list
+                elif attr == 'extra_headers':
+                    setter = self.process_extra_headers
                 else:
                     # AttributeError here? Your Payload subclass is missing a set_<attr> implementation
                     setter = getattr(self, 'set_%s' % attr)
@@ -278,6 +290,42 @@ class BasePayload(object):
             raise AnymailUnsupportedFeature("%s does not support %s" % (self.esp_name, feature),
                                             email_message=self.message, payload=self, backend=self.backend)
 
+    def process_extra_headers(self, headers):
+        # Handle some special-case headers, and pass the remainder to set_extra_headers.
+        # (Subclasses shouldn't need to override this.)
+        headers = CaseInsensitiveDict(headers)  # email headers are case-insensitive per RFC-822 et seq
+
+        reply_to = headers.pop('Reply-To', None)
+        if reply_to:
+            # message.extra_headers['Reply-To'] will override message.reply_to
+            # (because the extra_headers attr is processed after reply_to).
+            # This matches the behavior of Django's EmailMessage.message().
+            self.set_reply_to(parse_address_list([reply_to], field="extra_headers['Reply-To']"))
+
+        if 'From' in headers:
+            # If message.extra_headers['From'] is supplied, it should override message.from_email,
+            # but message.from_email should be used as the envelope_sender. See:
+            #   - https://code.djangoproject.com/ticket/9214
+            #   - https://github.com/django/django/blob/1.8/django/core/mail/message.py#L269
+            #   - https://github.com/django/django/blob/1.8/django/core/mail/backends/smtp.py#L118
+            header_from = parse_address_list(headers.pop('From'), field="extra_headers['From']")
+            envelope_sender = parse_single_address(self.message.from_email, field="from_email")  # must be single
+            self.set_from_email_list(header_from)
+            self.set_envelope_sender(envelope_sender)
+
+        if 'To' in headers:
+            # If message.extra_headers['To'] is supplied, message.to is used only as the envelope
+            # recipients (SMTP.sendmail to_addrs), and the header To is spoofed. See:
+            #   - https://github.com/django/django/blob/1.8/django/core/mail/message.py#L270
+            #   - https://github.com/django/django/blob/1.8/django/core/mail/backends/smtp.py#L119-L120
+            # No current ESP supports this, so this code is mainly here to flag
+            # the SMTP backend's behavior as an unsupported feature in Anymail:
+            header_to = headers.pop('To')
+            self.set_spoofed_to_header(header_to)
+
+        if headers:
+            self.set_extra_headers(headers)
+
     #
     # Attribute validators
     #
@@ -380,6 +428,7 @@ class BasePayload(object):
         self.unsupported_feature('reply_to')
 
     def set_extra_headers(self, headers):
+        # headers is a CaseInsensitiveDict, and is a copy (so is safe to modify)
         self.unsupported_feature('extra_headers')
 
     def set_text_body(self, body):
@@ -411,7 +460,15 @@ class BasePayload(object):
         raise NotImplementedError("%s.%s must implement add_attachment or set_attachments" %
                                   (self.__class__.__module__, self.__class__.__name__))
 
+    def set_spoofed_to_header(self, header_to):
+        # In the unlikely case an ESP supports *completely replacing* the To message header
+        # without altering the actual envelope recipients, the backend can implement this.
+        self.unsupported_feature("spoofing `To` header")
+
     # Anymail-specific payload construction
+    def set_envelope_sender(self, email):
+        self.unsupported_feature("envelope_sender")
+
     def set_metadata(self, metadata):
         self.unsupported_feature("metadata")
 
@@ -439,3 +496,29 @@ class BasePayload(object):
     # ESP-specific payload construction
     def set_esp_extra(self, extra):
         self.unsupported_feature("esp_extra")
+
+    #
+    # Helpers for concrete implementations
+    #
+
+    def serialize_json(self, data):
+        """Returns data serialized to json, raising appropriate errors.
+
+        Essentially json.dumps with added context in any errors.
+
+        Useful for implementing, e.g., serialize_data in a subclass,
+        """
+        try:
+            return json.dumps(data, default=self._json_default)
+        except TypeError as err:
+            # Add some context to the "not JSON serializable" message
+            raise AnymailSerializationError(orig_err=err, email_message=self.message,
+                                            backend=self.backend, payload=self)
+
+    @staticmethod
+    def _json_default(o):
+        """json.dump default function that handles some common Payload data types"""
+        if isinstance(o, CaseInsensitiveDict):  # used for headers
+            return dict(o)
+        raise TypeError("Object of type '%s' is not JSON serializable" %
+                        o.__class__.__name__)
