@@ -1,12 +1,9 @@
-import json
-
 import requests
-# noinspection PyUnresolvedReferences
 from six.moves.urllib.parse import urljoin
 
 from anymail.utils import get_anymail_setting
 from .base import AnymailBaseBackend, BasePayload
-from ..exceptions import AnymailRequestsAPIError, AnymailSerializationError
+from ..exceptions import AnymailRequestsAPIError
 from .._version import __version__
 
 
@@ -149,15 +146,3 @@ class RequestsPayload(BasePayload):
     def serialize_data(self):
         """Performs any necessary serialization on self.data, and returns the result."""
         return self.data
-
-    def serialize_json(self, data):
-        """Returns data serialized to json, raising appropriate errors.
-
-        Useful for implementing serialize_data in a subclass,
-        """
-        try:
-            return json.dumps(data)
-        except TypeError as err:
-            # Add some context to the "not JSON serializable" message
-            raise AnymailSerializationError(orig_err=err, email_message=self.message,
-                                            backend=self.backend, payload=self)
