@@ -1,13 +1,13 @@
+import uuid
 from email.utils import quote as rfc822_quote
 import warnings
 
-from django.core.mail import make_msgid
 from requests.structures import CaseInsensitiveDict
 
 from .base_requests import AnymailRequestsBackend, RequestsPayload
 from ..exceptions import AnymailConfigurationError, AnymailRequestsAPIError, AnymailWarning
 from ..message import AnymailRecipientStatus
-from ..utils import BASIC_NUMERIC_TYPES, get_anymail_setting, timestamp, update_deep, parse_address_list
+from ..utils import BASIC_NUMERIC_TYPES, get_anymail_setting, timestamp, update_deep
 
 
 class EmailBackend(AnymailRequestsBackend):
@@ -99,43 +99,19 @@ class SendGridPayload(RequestsPayload):
         """Performs any necessary serialization on self.data, and returns the result."""
 
         if self.generate_message_id:
-            self.ensure_message_id()
+            self.set_anymail_id()
         self.build_merge_data()
 
-        headers = self.data["headers"]
-        if "Reply-To" in headers:
-            # Reply-To must be in its own param
-            reply_to = headers.pop('Reply-To')
-            self.set_reply_to(parse_address_list([reply_to]))
-        if len(headers) > 0:
-            self.data["headers"] = dict(headers)  # flatten to normal dict for json serialization
-        else:
+        if not self.data["headers"]:
             del self.data["headers"]  # don't send empty headers
 
         return self.serialize_json(self.data)
 
-    def ensure_message_id(self):
-        """Ensure message has a known Message-ID for later event tracking"""
-        if "Message-ID" not in self.data["headers"]:
-            # Only make our own if caller hasn't already provided one
-            self.data["headers"]["Message-ID"] = self.make_message_id()
-        self.message_id = self.data["headers"]["Message-ID"]
+    def set_anymail_id(self):
+        """Ensure message has a known anymail_id for later event tracking"""
 
-        # Workaround for missing message ID (smtp-id) in SendGrid engagement events
-        # (click and open tracking): because unique_args get merged into the raw event
-        # record, we can supply the 'smtp-id' field for any events missing it.
-        self.data.setdefault("custom_args", {})["smtp-id"] = self.message_id
-
-    def make_message_id(self):
-        """Returns a Message-ID that could be used for this payload
-
-        Tries to use the from_email's domain as the Message-ID's domain
-        """
-        try:
-            _, domain = self.data["from"]["email"].split("@")
-        except (AttributeError, KeyError, TypeError, ValueError):
-            domain = None
-        return make_msgid(domain=domain)
+        self.message_id = str(uuid.uuid4())
+        self.data.setdefault("custom_args", {})["anymail_id"] = self.message_id
 
     def build_merge_data(self):
         """Set personalizations[...]['substitutions'] and data['sections']"""
